open Okauth_crypto

module Type = struct
  type 's config = {
    (** Base external URL of the application *)
    base_url: string;

    (** Display name of the application *)
    app_name: string;

    (** App's public encryption key *)
    public_encryption_key: Public_key.t;

    (** App's secret encryption key *)
    secret_encryption_key: Secret_key.t;

    (** App's public signing key *)
    public_signing_key: Public_key.t;

    (** App's secret signing key *)
    secret_signing_key: Secret_key.t;

    (** Session storage *)
    storage: 's;
  }

  module type Storage = sig
    type t

    (** Returns value of the given [~key] if it exists in the storage *)
    val find: t -> key:string -> string option

    (** Sets the value of the given [~key] to [~value] *)
    val set: t -> key:string -> value:string -> unit

    (** Removes the value of the given [~key] *)
    val remove: t -> key:string -> unit
  end

  module type S = sig
    type storage
    (** Middleware for user authentication via Okauth. *)
    val middleware: storage config -> Okweb.Middleware.t
  end
end

module type S = sig
  include module type of Type

  module In_memory_storage: sig
    include Storage

    (** Creates new empty in-memory storage *)
    val empty: t

    val size: t -> int
  end

  [@@@warning "-67"]
  module Make(Clt: module type of Okweb.Client)(St: Storage): S with type storage := St.t
end

